/* 
 * tclWinFile.c --
 *
 *      This file contains temporary wrappers around UNIX file handling
 *      functions. These wrappers map the UNIX functions to Win32 HANDLE-style
 *      files, which can be manipulated through the Win32 console redirection
 *      interfaces.
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

static char sccsid[] = "@(#) tclWinFile.c 1.6 95/09/11 20:17:01";

#include <sys/stat.h>
#include "tclInt.h"
#include "tclPort.h"

/*
 * The following table is used to keep track of currently open temporary files.
 */

static Tcl_HashTable *tempFileHashPtr = NULL;

static int 		DoCreate _ANSI_ARGS_((char *fileName, DWORD mode,
			    DWORD creation, DWORD attrs, int flags));

/*
 *----------------------------------------------------------------------
 *
 * TclCreateTempFile --
 *
 *	This function opens a Win32 file with the property that it will be
 *      deleted when its file handle is closed.
 *
 * Results:
 *	A handle to a Win32 file
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int
TclCreateTempFile(fileName, flags, track)
    char *fileName;
    int flags;
    int track;			/* 1 if the file will be deleted by Tcl,
				 * else 0. */
{
    SECURITY_ATTRIBUTES secAttr;
    DWORD mode = GENERIC_READ;
    int oflags = O_TEXT | O_RDONLY;
    HANDLE handle;
    int fd;
    DWORD atts;

    if(flags & O_WRONLY) {
	mode = GENERIC_WRITE;
	oflags = O_TEXT | O_APPEND;
    }
    if(flags & O_RDWR) {
	mode = GENERIC_READ | GENERIC_WRITE;
	oflags = O_TEXT;
    }
    
    secAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    secAttr.lpSecurityDescriptor = NULL;
    secAttr.bInheritHandle = FALSE;

    atts = (track ? FILE_ATTRIBUTE_NORMAL :
	    FILE_ATTRIBUTE_NORMAL | FILE_FLAG_DELETE_ON_CLOSE);
	  
    handle = CreateFile(fileName, mode, FILE_SHARE_READ | FILE_SHARE_WRITE,
	    &secAttr, OPEN_ALWAYS, atts, 0);
    if (handle != INVALID_HANDLE_VALUE) {
	fd = _open_osfhandle((long) handle, oflags);

	/*
	 * We only enter the file into the hash table if it is being newly
	 * created.
	 */

	if ((fd >= 0) && (flags & O_CREAT) && track) {
	    Tcl_HashEntry *hashPtr;
	    int new;
	    char *name = ckalloc(strlen(fileName)+1);
	    strcpy(name, fileName);

	    if (tempFileHashPtr == NULL) {
		tempFileHashPtr =
		    (Tcl_HashTable*) ckalloc(sizeof(Tcl_HashTable));
		Tcl_InitHashTable(tempFileHashPtr, TCL_ONE_WORD_KEYS);
	    }
	    hashPtr = Tcl_CreateHashEntry(tempFileHashPtr, (char *)fd, &new);
	    if (!new) {
		panic("duplicate temporary file descriptor");
	    }
	    Tcl_SetHashValue(hashPtr, name);
	}
	return fd;
    } else {
	return -1;
    }
}

/*
 *----------------------------------------------------------------------
 *
 * TclCheckTempFile --
 *
 *	This function calls TclWinLseek() to rewind to the beginning of a
 *      temporary file.
 *
 * Results:
 *	0 on success, -1 on failure.
 *
 * Side effects:
 *	Rewinds to the beginning of the file.
 *
 *----------------------------------------------------------------------
 */

int
TclCheckTempFile(inputId, fileName)
    int inputId;
    char *fileName;
{
    return lseek(inputId, 0L, 0);
}

/*
 *----------------------------------------------------------------------
 *
 * TclRemoveTempFile --
 *
 *	This function looks up a temporary file by file descriptor
 *	and removes it from the hash table and the disk.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Removes the specified temporary file.
 *
 *----------------------------------------------------------------------
 */

void
TclRemoveTempFile(fd)
    int fd;
{
    char *name;
    Tcl_HashEntry *entryPtr = Tcl_FindHashEntry(tempFileHashPtr, (char *)fd);
    if (entryPtr != NULL) {
	name = (char *)Tcl_GetHashValue(entryPtr);
	unlink(name);
	ckfree(name);
	Tcl_DeleteHashEntry(entryPtr);
    }
}

/*
 *----------------------------------------------------------------------
 *
 * DoCreate --
 *
 *	This function creates a Win32 file.
 *
 * Results:
 *	Win32 file handle
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
DoCreate(fileName, mode, creation, attrs, flags)
    char *fileName;
    DWORD mode;
    DWORD creation;
    DWORD attrs; 
    int flags;
{
    HANDLE val;
    SECURITY_ATTRIBUTES secAttr;
    BOOL isDeleteOnClose;

    isDeleteOnClose = attrs & FILE_FLAG_DELETE_ON_CLOSE;
    secAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    secAttr.lpSecurityDescriptor = NULL;
    secAttr.bInheritHandle = FALSE;

    val = CreateFile(fileName, mode, FILE_SHARE_READ | FILE_SHARE_WRITE,
	    &secAttr, creation,
	    (flags & O_TRUNC && isDeleteOnClose ?
		    attrs ^ FILE_FLAG_DELETE_ON_CLOSE : attrs), 0);
    
    if(val == INVALID_HANDLE_VALUE) {
	return(-1);
    }

    if(flags & O_TRUNC) {

	/*
	 * Now we know the file exists so close the handle and truncate the
	 * file.
	 */ 

	CloseHandle(val);
	val = CreateFile(fileName, mode, FILE_SHARE_READ | FILE_SHARE_WRITE,
		&secAttr, TRUNCATE_EXISTING, attrs, 0);
    }
    
    if(flags & O_APPEND) {
	SetFilePointer(val, 0, NULL, FILE_END);
    }

    return((int)val);
}

/*
 *----------------------------------------------------------------------
 *
 * TclWinRead --
 *
 *	This function is a wrapper for the normal read() call.  It
 *	works around a bug that causes read to return an error when
 *	the write side of a pipe is closed.
 *
 * Results:
 *	Number of bytes read on success, 0 on eof, and -1 on failure.
 *
 * Side effects:
 *	Reads data from file indicated by fd.
 *
 *----------------------------------------------------------------------
 */

#undef read

int
TclWinRead(fd, buf, num)
    int fd;
    const void *buf;
    int num;
{
    int count = read(fd, buf, num);
    if ((count < 0) && (errno == EPIPE)) {
	return 0;
    }
    return count;
}

