/* 
 * tclWinInit.c --
 *
 *	Contains the Windows-specific interpreter initialization functions.
 *
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "tclInt.h"
#include "tclPort.h"
#include <winreg.h>


/*
 *----------------------------------------------------------------------
 *
 * TclPlatformInit --
 *
 *	Performs Windows-specific interpreter initialization related to the
 *	tcl_library variable.  Also sets up the HOME environment variable
 *	if it is not already set.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Sets "tcl_library" and "env(HOME)" Tcl variables
 *
 *----------------------------------------------------------------------
 */

void
TclPlatformInit(interp)
    Tcl_Interp *interp;
{
    char *ptr;
    HKEY regKey;
    long size = MAX_PATH;
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    
    /*
     * Look in the environment and the registry for a TCL_LIBRARY
     * entry.  Store the result in the dynamic string.
     */

    ptr = Tcl_GetVar2(interp, "env", "TCL_LIBRARY", TCL_GLOBAL_ONLY);
    if (ptr == NULL) {
	Tcl_DStringAppend(&ds, "Software\\SUNW\\Tcl\\", -1);
	Tcl_DStringAppend(&ds, TCL_VERSION, -1);

	/*
	 * Windows 3.1 only supports HKEY_CLASSES_ROOT.
	 */

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, Tcl_DStringValue(&ds), 0,
		KEY_READ, &regKey) != ERROR_SUCCESS) {
	    RegOpenKeyEx(HKEY_CLASSES_ROOT, Tcl_DStringValue(&ds), 0,
		    KEY_READ, &regKey);
	}
	Tcl_DStringSetLength(&ds, size);

	/*
	 * Since Win32s doesn't support multiple valued keys, we have to try
	 * using the name as a value name and as a subkey.
	 */

	if ((RegQueryValueEx(regKey, "TCL_LIBRARY", NULL, NULL,
		Tcl_DStringValue(&ds), &size) == ERROR_SUCCESS)
		|| (RegQueryValue(regKey, "TCL_LIBRARY", Tcl_DStringValue(&ds),
			&size) == ERROR_SUCCESS)) {
	    RegCloseKey(regKey);
	    Tcl_DStringSetLength(&ds, size);
	} else {
	    Tcl_DStringSetLength(&ds, 0);
	}
    } else {
	Tcl_DStringSetLength(&ds, 0);
	Tcl_DStringAppend(&ds, ptr, -1);
    }

    /*
     * If the path doesn't exist, look for the library relative to the
     * tcl.dll library.
     */

    if ((Tcl_DStringLength(&ds) == 0)
	    || (access(Tcl_DStringValue(&ds), 0) != 0)) {
	char *lastSlash = NULL;
	char libName[13];	/* used to hold constructed library name */

	/*
	 * Path was invalid, so we need to construct a default path from the
	 * location of the executable.  Convert backslashes at the same time.
	 */

	Tcl_DStringSetLength(&ds, MAX_PATH);
	sprintf(libName, "tcl%d%d.dll", TCL_MAJOR_VERSION, TCL_MINOR_VERSION);
	Tcl_DStringSetLength(&ds, SearchPath(NULL, libName, NULL, MAX_PATH,
		Tcl_DStringValue(&ds), &lastSlash));
	if (Tcl_DStringLength(&ds) > 0) {
	    Tcl_DStringSetLength(&ds, lastSlash - Tcl_DStringValue(&ds));
	    Tcl_DStringAppend(&ds, "../lib/tcl", -1);
	    Tcl_DStringAppend(&ds, TCL_VERSION, -1);
	}
    }

    /*
     * Convert backslashes to slashes.
     */

    for (ptr = Tcl_DStringValue(&ds); *ptr != '\0'; ptr++) {
	if (*ptr == '\\') {
	    *ptr = '/';
	}
    }

    Tcl_SetVar(interp, "tcl_library", Tcl_DStringValue(&ds), TCL_GLOBAL_ONLY);

    /*
     * Set up the HOME environment variable.
     */

    ptr = Tcl_GetVar2(interp, "env", "HOME", TCL_GLOBAL_ONLY);
    if (ptr == NULL) {
	Tcl_SetVar2(interp, "env", "HOME", ".", TCL_GLOBAL_ONLY);
    }

    Tcl_DStringFree(&ds);
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_Init --
 *
 *	This procedure is typically invoked by Tcl_AppInit procedures
 *	to perform additional initialization for a Tcl interpreter,
 *	such as sourcing the "init.tcl" script.
 *
 * Results:
 *	Returns a standard Tcl completion code and sets interp->result
 *	if there is an error.
 *
 * Side effects:
 *	Depends on what's in the init.tcl script.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_Init(interp)
    Tcl_Interp *interp;		/* Interpreter to initialize. */
{
    static char initCmd[] =
	"if [file exists [info library]/init.tcl] {\n\
	    source [info library]/init.tcl\n\
	} else {\n\
	    set msg \"can't find [info library]/init.tcl; perhaps you \"\n\
	    append msg \"need to\\ninstall Tcl or set your TCL_LIBRARY \"\n\
	    append msg \"environment variable?\"\n\
	    error $msg\n\
	}";

    return Tcl_Eval(interp, initCmd);
}
